#include <stdio.h>
#include <stdlib.h>
#include <zlib.h>
#include "menu.h"
#include "gbcore/gb.h"
#include "fatfs/tff.h"

uLong sram_crc;

int save_sram(byte *buf,int size)
{
	int ret=1;
	FIL fp;
	WORD bytes_write;
	const int sram_tbl[]={1,1,1,4,16,8};
	int sram_size = 0x2000 * sram_tbl[size];

	if (rom_get_info()->cart_type>=0x0f && rom_get_info()->cart_type<=0x13){
		int tmp = renderer_get_timer_state();
		memcpy(buf+sram_size, &tmp, sizeof(int));
		sram_size += sizeof(int);
	}

	uLong crc = crc32(0L, Z_NULL, 0);
	crc = crc32(crc, buf, sram_size);
	if (sram_crc == crc)
		return 1;
	
	sprintf(SavePath, "RINSAVE/%s.SAV", RomName);
	if(f_open(&fp, SavePath, FA_CREATE_ALWAYS | FA_WRITE) != FR_OK)
		return 0;
	
	cheat_decreate_cheat_map();
	if(f_write(&fp, buf, sram_size, &bytes_write) != FR_OK)
		ret = 0;
	if(bytes_write != sram_size)
		ret = 0;
	cheat_create_cheat_map();
	
	f_close(&fp);

	if(ret)
		sram_crc = crc;
	
	return ret;
}

int load_sram(byte *buf, int bufsize)
{
	FIL fp;
	FILINFO finfo;
	WORD bytes_read;
	
	memset(buf, 0, bufsize);
	sram_crc = crc32(0L, Z_NULL, 0);
	
	sprintf(SavePath, "RINSAVE/%s.SAV", RomName);
	
	if(f_stat(SavePath, &finfo) != FR_OK)
		return 0;
	if(finfo.fsize > bufsize)
		return 0;
	if(f_open(&fp, SavePath, FA_READ) != FR_OK)
		return 0;
	
	int ramsize=0;
	while(ramsize < finfo.fsize)
	{
		if(f_read(&fp, buf + ramsize, 0x200, &bytes_read) != FR_OK){
			f_close(&fp);
			return 0;
		}
		if(bytes_read == 0)
			break;
		ramsize += bytes_read;
	}
	f_close(&fp);
	
	if(ramsize & 4)
		renderer_set_timer_state(*(int*)(buf+ramsize-4));
	
	sram_crc = crc32(sram_crc, buf, ramsize);
	
	return ramsize;
}
